//--------------------------------------------------------------------------
//
// @function CalcHistDemo.cpp
// Display the source image.
// Display RGB planes (layers) and histograms.
// Display HSV planes (layers) and histograms.
// 
//--------------------------------------------------------------------------

#include "opencv2/highgui/highgui.hpp"
#include "opencv2/imgproc/imgproc.hpp"
#include <iostream>

using namespace std;
using namespace cv;

int main( int argc, char** argv )
{
	//--------------------------------------------------------------------------
	// Step 1. Initialization - prepare the source image img_BGR and display it
	//--------------------------------------------------------------------------
	// Load input_image, check for errors
	Mat input_image;

	if ( argc != 2 )
	{ 
		cout << "Usage: ./CalcHistDemo <input_image>" << endl;
		return -1; 
	}
	else 
	{	
		input_image = imread( argv[ 1 ], 1 );
		if( ! input_image.data )
		{
			cout << "Usage: ./CalcHistDemo <input_image>" << endl;
			return -1;
		}
	}

	// Scale-down the image to the size img_width x img_height.
	// (If we'll concatenate (horizontally) the three layers of the image 
	// into one large "merged" image, this image has 
	// to fit the screen of the notebook)
	int img_width = 400, img_height = 400;
	Mat img_BGR = Mat::zeros( img_height, img_width, input_image.type( ) );
	resize( input_image, img_BGR, img_BGR.size( ), 0, 0 ); // img_BGR will be our "source image"

	// Display the source image img_BGR
	namedWindow( "Source Image", CV_WINDOW_AUTOSIZE );
	imshow(      "Source Image", img_BGR );

	// -----------------------RGB case------------------------------------------

	//--------------------------------------------------------------------------
	// Step 2. Extract the R, G, B layers 
	// and display them in one large "merged" image
	// (similarly to plotting together several Matlab Subplots in one Figure)
	//--------------------------------------------------------------------------

	// Extract the R, G, B layers
	vector<Mat> img_planes_BGR;
	split( img_BGR, img_planes_BGR );

	Mat img_R, img_G, img_B;
	img_planes_BGR[ 2 ].copyTo( img_R );
	img_planes_BGR[ 1 ].copyTo( img_G );
	img_planes_BGR[ 0 ].copyTo( img_B );

	// Concatenate R, G, B layers together into one large "merged" image
	// (similarly to plotting together several Matlab Subplots in one Figure)
	Mat three_layers_RGB = Mat::zeros( img_height, 3 * img_width, CV_8UC1 );

	Mat ROI_R, ROI_G, ROI_B;
	ROI_R = three_layers_RGB.colRange( 0            ,     img_width );
	img_R.copyTo( ROI_R );
	ROI_G = three_layers_RGB.colRange(     img_width, 2 * img_width );
	img_G.copyTo( ROI_G );
	ROI_B = three_layers_RGB.colRange( 2 * img_width, 3 * img_width );
	img_B.copyTo( ROI_B );

	// Display the R, G, B layers using the "merged" image
	namedWindow( "R, G, B layers", CV_WINDOW_AUTOSIZE );
	imshow(      "R, G, B layers", three_layers_RGB );

	//--------------------------------------------------------------------------
	// Step 3. Compute the histograms of the R, G, B layers and plot them in one
	// "merged" image
	//--------------------------------------------------------------------------

	// Set the "size" of the histograms (the number of "gray levels" 
	// on the X axis)
	const int histSize_R = 256; // 256 different "gray levels"
	const int histSize_G = 256;
	const int histSize_B = 256;

	// Set the "range" of the histograms (minimal and maximal values on the X axis)
	float range_R[ ] = { 0, 256 }; // [0,...,255]
	float range_G[ ] = { 0, 256 };
	float range_B[ ] = { 0, 256 }; 
	const float* histRange_R = { range_R };
	const float* histRange_G = { range_G };
	const float* histRange_B = { range_B };

	// Compute the histograms of the R, G, B layers
	Mat hist_R, hist_G, hist_B;
	calcHist( &img_R, 1, 0, Mat( ), hist_R, 1, &histSize_R, &histRange_R, true, false );
	calcHist( &img_G, 1, 0, Mat( ), hist_G, 1, &histSize_G, &histRange_G, true, false );
	calcHist( &img_B, 1, 0, Mat( ), hist_B, 1, &histSize_B, &histRange_B, true, false );

	// Prepare the image with histograms
	int hist_width = 512, hist_height = 512;
	int bin_width = 2;

	// Normalize the histograms to the range [ 0, hist_height ]
	normalize( hist_R, hist_R, 0, hist_height, NORM_MINMAX, -1, Mat( ) );
	normalize( hist_G, hist_G, 0, hist_height, NORM_MINMAX, -1, Mat( ) );
	normalize( hist_B, hist_B, 0, hist_height, NORM_MINMAX, -1, Mat( ) );

	// Plot the histograms of R, G, B layers in one image
	Mat three_histograms_RGB( hist_width, hist_height, CV_8UC3, Scalar( 0, 0, 0 ) );

	for( int i = 1; i < histSize_R; i++ ) // R layer
		line( three_histograms_RGB,
			  Point( bin_width * ( i - 1 ), hist_height - cvRound( hist_R.at<float>( i - 1 ) ) ), 
		      Point( bin_width *   i      , hist_height - cvRound( hist_R.at<float>( i     ) ) ), 
	          Scalar( 0, 0, 255 ), 2, 8, 0  );
	
	for( int i = 1; i < histSize_G; i++ ) // G layer
		line( three_histograms_RGB,
			  Point( bin_width * ( i - 1 ), hist_height - cvRound( hist_G.at<float>( i - 1 ) ) ), 
		      Point( bin_width *   i      , hist_height - cvRound( hist_G.at<float>( i     ) ) ), 
	          Scalar( 0, 255, 0 ), 2, 8, 0  ); 
	
	for( int i = 1; i < histSize_B; i++ ) // B layer
		line( three_histograms_RGB,
			  Point( bin_width * ( i - 1 ), hist_height - cvRound( hist_B.at<float>( i - 1 ) ) ), 
		      Point( bin_width *   i      , hist_height - cvRound( hist_B.at<float>( i     ) ) ), 
	          Scalar( 255, 0, 0 ), 2, 8, 0  );       


	// Display the histograms of R, G, B layers in one image
	namedWindow( "Histograms of R, G, B layers", CV_WINDOW_AUTOSIZE );
	imshow(      "Histograms of R, G, B layers", three_histograms_RGB );

	// -----------------------HSV case------------------------------------------

	//--------------------------------------------------------------------------
	// Step 4. Convert the source image img_BGR from BGR color space 
	// to HSV color space.
	//--------------------------------------------------------------------------
	
	// ...

	//--------------------------------------------------------------------------
	// Step 5. Extract the H, S, V layers 
	// and display them in one large "merged" image
	// (similarly to plotting together several Matlab Subplots in one Figure)
	//--------------------------------------------------------------------------
	
	// Extract the H, S, V layers
	
	// ...

	// Concatenate H, S, V layers together into one large "merged" image
	// (similarly to plotting together several Matlab Subplots in one Figure)
	
	// ...

	// Correction for the display of H layer 
	// (after the computation of three_layers_HSV and before its display 
	// in a window)

	// double scale = (double) 255 / 179;
	// three_layers_HSV.colRange( 0, img_width ) = three_layers_HSV.colRange( 0, img_width ) * scale;
	
	// Display the H, S, V layers using the "merged" image
	
	// ...

	//--------------------------------------------------------------------------
	// Step 6. Compute the histograms of the H, S, V layers and plot them in one
	// "merged" image
	//--------------------------------------------------------------------------

	// Set the "size" of the histograms (the number of "gray levels" 
	// on the X axis)

	// const int histSize_H = 180; // Correction for the "size" of the H layer:
								   // 180 different "gray levels"
	
	// ...

	// Set the "range" of the histograms (minimal and maximal values on the X axis)

	// float range_H[ ] = { 0, 180 }; // Correction for the "range" of the H layer:
								      // [0,...,179]
	
	// ...

	// Compute the histograms of the H, S, V layers
	
	// ...

	/// Normalize the histograms to the range [ 0, hist_height ]
	
	// ...

	// Plot the histograms of H, S, V layers in one image
	
	// ...

	// Display the histograms of H, S, V layers in one image
	
	// ...

	//--------------------------------------------------------------------------
	// Step 7. Finalization
	//--------------------------------------------------------------------------
	waitKey( 0 ); // Wait for a pressed key

	return 0;
}
